<?php
// $Id: FeedsSelfNodeProcessor.inc,v 1.2.2.1 2011/02/01 08:19:21 twistor Exp $

/**
 * @file
 * Class definition of FeedsSelfNodeProcessor.
 */

/**
 * Allows a feed node to populate fields on itself.
 */
class FeedsSelfNodeProcessor extends FeedsNodeProcessor {

  /**
   * Process the result of the parsing stage.
   *
   * This will only take the first item and parse it. Ignoring the other items.
   *
   * @param FeedsSource $source
   *   Source information about this import.
   * @param FeedsParserResult $parser_result
   *   The result of the parsing stage.
   */
  public function process(FeedsImportBatch $batch, FeedsSource $source) {
    $item = $batch->shiftItem();
    $hash = $this->hash($item);

    if ($hash == $this->getHash($source->feed_nid)) {
      return;
    }
    $node = node_load($source->feed_nid);
    $node->log = 'Updated by FeedsSelfNodeProcessor';
    $node->changed = FEEDS_REQUEST_TIME;
    $node->format = $this->config['input_format'];
    $node->feeds_node_item = new stdClass();
    $node->feeds_node_item->nid = $node->nid;
    $node->feeds_node_item->id = $this->id;
    $node->feeds_node_item->feed_nid = $source->feed_nid;
    $node->feeds_node_item->imported = FEEDS_REQUEST_TIME;
    $node->feeds_node_item->url = '';
    $node->feeds_node_item->guid = '';
    $node->feeds_node_item->hash = $hash;

    try {
      $this->map($batch, $node);
      module_load_include('inc', 'node', 'node.pages');
      node_object_prepare($node);
      /**
       * The feed never gets added to the feeds_node_item table by itself.
       */
      if (!db_fetch_object(db_query("SELECT feed_nid FROM {feeds_node_item} WHERE nid = %d", $node->nid))) {
        drupal_write_record('feeds_node_item', $node->feeds_node_item);
      }
      node_save($node);
    }
    catch (Exception $e) {
      drupal_set_message($e->getMessage(), 'warning');
      watchdog('feeds', $e->getMessage(), array(), WATCHDOG_WARNING);
      return;
    }
    drupal_set_message(t('@type %title has been updated.', array('@type' => node_get_types('name', $node), '%title' => $node->title)));
  }

  /**
   * Override parent::configDefaults().
   */
  public function configDefaults() {
    $defaults = parent::configDefaults();
    $defaults['expire'] = FEEDS_EXPIRE_NEVER;
    return $defaults;
  }

  /**
   * Override parent::configForm().
   */
  public function configForm(&$form_state) {
    $form = parent::configForm($form_state);
    unset($form['expire']);
    unset($form['update_existing']);
    unset($form['content_type']);
    unset($form['author']);
    return $form;
  }
}
